/*
 * Copyright  2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 *  
 */

package com.sunlabs.archsim.fleet.ships.stride;

import java.util.HashMap;
import java.util.Map;

import com.sunlabs.archsim.base.common.Parameters;
import com.sunlabs.archsim.base.simulation.Delay;
import com.sunlabs.archsim.fleet.common.Data;
import com.sunlabs.archsim.fleet.common.Payload;
import com.sunlabs.archsim.gasp.Action;
import com.sunlabs.archsim.gasp.Component;
import com.sunlabs.archsim.gasp.Function;
import com.sunlabs.archsim.gasp.Terminal;
import com.sunlabs.archsim.gasp.TriggerAction;
import com.sunlabs.archsim.gasp.terminals.InputTerminal;
import com.sunlabs.archsim.gasp.terminals.OutputTerminal;


class StrideTestSource extends Component {

	// terminal names, <output, input > pairs
	public static final String START_VALUE= "startValue";
	public static final String ACK_START_VALUE= "ackStartValue";
	
	public static final String STEP_SIZE= "stepSize";
	public static final String ACK_STEP_SIZE= "ackStepSize";
	
	public static final String NO_STEPS= "numberOfSteps";
	public static final String ACK_NO_STEPS= "ackNumberOfSteps";

	public static final String ACK_DONE= "ackDone";
	public static final String DONE= "done";

	// number of cycles
	int noCycles= 10;
	
	// start value for each cycle
	// exercise regular behavior, and boundary cases
	Data[] startValue= { 	
			new Data(0, true),
			new Data(1000, true),
			new Data(10, true), 
			new Data(1, false), // invalid data
			new Data(1, true),
			new Data(1, true),
			new Data(StrideDefaults.MAX_COUNTER+1, true), // out of bound start
			new Data(StrideDefaults.MIN_COUNTER-1, true), // out of bounds start
			new Data(StrideDefaults.MAX_COUNTER-5, true), // close to MAX
			new Data(StrideDefaults.MIN_COUNTER+5, true) // close to MIN
		};
	
	// step sizes for each cycle
	Data[] stepSize= {
			new Data(1, true),
			new Data(-100, true),
			new Data(2, true),
			new Data(1, true),
			new Data(1, false), // invalid data
			new Data(1, true),
			new Data(1, true),
			new Data(1, true),
			new Data(2, true), // to go over MAX
			new Data(-2, true) // to go under MIN
			
	};
	
	// number of steps for each cycle
	Data[] noSteps= {
			new Data(10, true),
			new Data(5, true),
			new Data(0, true),
			new Data(1, true),
			new Data(1, true),
			new Data(1, false),
			new Data(1, true),
			new Data(1, true),
			new Data(10, true), // drive accross MAX
			new Data(10, true) // drive accross MIN
	};
		
	// StrideSourceAction
	Action strideSourceAction;
	
	// the terminals
	private Terminal startValueTerm;
	private Terminal ackStartValueTerm;
	private Terminal stepSizeTerm;
	private Terminal ackStepSizeTerm;
	private Terminal noStepsTerm;
	private Terminal ackNoStepsTerm;
	private Terminal ackDoneTerm;
	private Terminal doneTerm;
	
	Map<String, Terminal> terminalNameMap;
	
	public StrideTestSource(String name, Parameters params) {
        super(name, null);
        
		// process the parameters
		if (params != null) {
			for (int i= 0; i<params.size(); i++) {
				String pName= params.getName(i);
				String sValue= params.getValue(i);
		
				director.error("Component: " + name + ", id= " + getID() 
						+ ": Attribute " + pName + " with value " + sValue
						+ " is ignored");					
		
			} // for
		} // if params != null
		
		buildStrideTestSource();

	} // constructor
	
	public StrideTestSource() {
		super("StrideSource", null);
		buildStrideTestSource();

	} // constructor
	
	
	
	protected void buildStrideTestSource() {
		terminalNameMap= new HashMap<String, Terminal>();
		
		// make the action
		strideSourceAction= new StrideTestSourceAction();
		addMember(strideSourceAction);
		
		// make the terminals, form the map
		startValueTerm= new OutputTerminal(START_VALUE, this);
		terminalNameMap.put(START_VALUE, startValueTerm);
		
		ackStartValueTerm= new InputTerminal(ACK_START_VALUE, this);
		terminalNameMap.put(ACK_START_VALUE, ackStartValueTerm);
		
		stepSizeTerm= new OutputTerminal(STEP_SIZE, this);
		terminalNameMap.put(STEP_SIZE, stepSizeTerm);
		
		ackStepSizeTerm= new InputTerminal(ACK_STEP_SIZE, this);
		terminalNameMap.put(ACK_STEP_SIZE, ackStepSizeTerm);
		
		noStepsTerm= new OutputTerminal(NO_STEPS, this);
		terminalNameMap.put(NO_STEPS, noStepsTerm);
		
		ackNoStepsTerm= new InputTerminal(ACK_NO_STEPS, this);
		terminalNameMap.put(ACK_NO_STEPS, ackNoStepsTerm);
		
		ackDoneTerm= new OutputTerminal(ACK_DONE, this);
		terminalNameMap.put(ACK_DONE, ackDoneTerm);
		
		doneTerm= new InputTerminal(DONE, this);
		terminalNameMap.put(DONE, doneTerm);
		
		// attach the terminals to the action
		strideSourceAction.attach(startValueTerm);
		strideSourceAction.attach(ackStartValueTerm);
		strideSourceAction.attach(stepSizeTerm);
		strideSourceAction.attach(ackStepSizeTerm);
		strideSourceAction.attach(noStepsTerm);
		strideSourceAction.attach(ackNoStepsTerm);
		strideSourceAction.attach(ackDoneTerm);
		strideSourceAction.attach(doneTerm);
		
		
	}
	
	public Terminal getTerminal(String name) {
		return terminalNameMap.get(name);
	}

	
	private class StrideTestSourceAction extends TriggerAction {
		
		public StrideTestSourceAction() {
			super("Stride source action");
			
			// make this action initially enabled
			enabled= true;
			
			outputFunctions.put(START_VALUE, new ValueFn(startValue));
			outputFunctions.put(STEP_SIZE, new ValueFn(stepSize));
			outputFunctions.put(NO_STEPS, new ValueFn(noSteps));
			outputFunctions.put(ACK_DONE, new AckFunction());
			
		} // StrideSourceAction constructor
		
		/* provide values for stride sources */
		private class ValueFn extends Function {
			//current cycle
			private int cycle;
			
			Data[] values;
			
			public ValueFn(Data[] v) {
				super("ValueFn");
				// remember the values
				values= v;
				// reset the cycle number
				cycle= 0;
				// determine function delay
				Integer fDelay= globals.intValue(StrideDefaults.STRIDE_ACK_DELAY);
				functionDelay= (fDelay != null)
						? new Delay(fDelay.intValue())
						: StrideDefaults.STRIDE_ACK_DELAY_DEF;
			}
			
			public Object value() {
				int index= cycle;
				cycle= (cycle+1);
				// are the test complete?
				if (cycle >= noCycles) {
					// get trigger delay
					Integer fDelay= globals.intValue(StrideDefaults.STRIDE_TRIGGER_DELAY);
					Delay triggerDelay= (fDelay != null)
							? new Delay(fDelay.intValue())
							: StrideDefaults.STRIDE_TRIGGER_DELAY_DEF;
					
					// schedule disabling
					director.schedule(disableTriggerCommand, null, triggerDelay);
				}
				Payload p= new Payload();
				p.data= values[index];
				// System.out.println("\nStart value= " + values[index].value + " valid= " + values[index].valid + "\n");
				return p;
			} // value
			
		} // ValueFn	
		
	} // class StrideTestSourceAction
	
} // class StrideSource
	

